@@ -1,5 +1,5 @@
 /*
- * FIG : Facility for Interactive Generation of figures
+ * FIG : Facility for Interactive Generation of figunres
  * Copyright (c) 1995 by Brian V. Smith
  *
  * The X Consortium, and any party obtaining a copy of these files from
@@ -8,11 +8,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -27,14 +30,15 @@
 static Boolean	create_n_write_xbm();
 
 Boolean
-write_xbm(file_name,mag)
+write_xbm(file_name,mag,margin)
     char	   *file_name;
     float	    mag;
+    int		    margin;
 {
     if (!ok_to_write(file_name, "EXPORT"))
 	return False;
 
-    return (create_n_write_xbm(file_name,mag));  /* write the xbm file */
+    return (create_n_write_xbm(file_name,mag/100.0,margin));  /* write the xbm file */
 }
 
 static Boolean	havegcs = False;
@@ -43,21 +47,24 @@ static unsigned long save_fg_color;
 static unsigned long save_bg_color;
 
 static Boolean
-create_n_write_xbm(filename,mag)
-    char	   *filename;
-    float	    mag;
+create_n_write_xbm(filename,mag,margin)
+    char	 *filename;
+    float	  mag;
+    int		  margin;
 {
-    int		    xmin, ymin, xmax, ymax;
-    int		    width, height;
-    Window	    sav_canvas;
-    int		    sav_objmask;
-    Pixmap	    largepm, bitmap;
-    int		    i;
-    float	    savezoom;
-    int		    savexoff, saveyoff;
-    int		    status;
-    Boolean	    zoomchanged;
-    GC		    xgc, gc_bitmap;
+    int		  xmin, ymin, xmax, ymax;
+    int		  width, height;
+    Window	  sav_canvas;
+    int		  sav_objmask;
+    Pixmap	  largepm, bitmap;
+    int		  i;
+    float	  savezoom;
+    int		  savexoff, saveyoff;
+    Boolean	  status;
+    int		  wrstat;
+    Boolean	  zoomchanged;
+    GC		  xgc, gc_bitmap;
+    int		  save_posn;
 
     /* this may take a while */
     set_temp_cursor(wait_cursor);
@@ -74,7 +81,11 @@ create_n_write_xbm(filename,mag)
     zoomxoff = zoomyoff = 0;
 
     /* Assume that there is at least one object */
+    /* save current point positioning grid and set to "ANY" */
+    save_posn = cur_pointposn;
+    cur_pointposn = P_ANY;
     compound_bound(&objects, &xmin, &ymin, &xmax, &ymax);
+    cur_pointposn = save_posn;
 
     /* adjust limits for magnification */
     xmin = round(xmin*zoomscale);
@@ -83,12 +94,12 @@ create_n_write_xbm(filename,mag)
     ymax = round(ymax*zoomscale);
 
     /* provide a small margin (pixels) */
-    if ((xmin -= 10) < 0)
+    if ((xmin -= margin) < 0)
 	xmin = 0;
-    if ((ymin -= 10) < 0)
+    if ((ymin -= margin) < 0)
 	ymin = 0;
-    xmax += 10;
-    ymax += 10;
+    xmax += margin;
+    ymax += margin;
 
     if (appres.DEBUG) {
 	elastic_box(xmin, ymin, xmax, ymax);
@@ -136,8 +147,7 @@ create_n_write_xbm(filename,mag)
     clear_pic_pixmaps();
 
     /* create pixmap from (0,0) to (xmax,ymax) */
-    largepm = XCreatePixmap(tool_d, canvas_win, xmax + 1, ymax + 1,
-			    DefaultDepthOfScreen(tool_s));
+    largepm = XCreatePixmap(tool_d, canvas_win, xmax + 1, ymax + 1, tool_dpth);
     /* clear it */
     XFillRectangle(tool_d, largepm, gccache[ERASE], 0, 0, xmax+1, ymax+1);
     sav_canvas = canvas_win;	/* save current canvas window id */
@@ -174,12 +184,15 @@ create_n_write_xbm(filename,mag)
 	sav_fill_gc[i] = fill_gc[i];
 	fill_gc[i] = xgc;
     }
-    if (XWriteBitmapFile(tool_d, filename, bitmap, width, height, -1, -1)
+    if ((wrstat=XWriteBitmapFile(tool_d, filename, bitmap, width, height, -1, -1))
 	!= BitmapSuccess) {
-	put_msg("Couldn't write xbm file");
+	if (wrstat==BitmapOpenFailed)
+	    file_msg("Couldn't write xbm file");
+	else if (wrstat==BitmapNoMemory)
+	    put_msg("not enough memory");
 	status = False;
     } else {
-	put_msg("Bitmap written to \"%s\"", filename);
+	put_msg("%dx%d XBM bitmap written to \"%s\"", width, height, filename);
 	status = True;
     }
     XFreePixmap(tool_d, largepm);
@@ -238,8 +251,8 @@ clear_compound_pic(compounds)
 clear_pic_pixmap(line)
     F_line	   *line;
 {
-    if (line->type == T_PIC_BOX && line->pic->pixmap != 0) {
-	free((char *) line->pic->pixmap);
+    if (line->type == T_PICTURE && line->pic->pixmap != 0) {
+	XFreePixmap(tool_d, line->pic->pixmap);
 	line->pic->pixmap = 0;
     }
 }
