@@ -9,11 +9,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -38,6 +41,9 @@ DeclareStaticArgs(14);
 static char	mousefun_l[MOUSEFUN_MAX];
 static char	mousefun_m[MOUSEFUN_MAX];
 static char	mousefun_r[MOUSEFUN_MAX];
+static char	mousefun_sh_l[MOUSEFUN_MAX];
+static char	mousefun_sh_m[MOUSEFUN_MAX];
+static char	mousefun_sh_r[MOUSEFUN_MAX];
 
 /* labels for the left and right buttons have 15 chars max */
 static char	lr_blank[] = "               ";
@@ -45,10 +51,15 @@ static char	lr_blank[] = "               ";
 /* give the middle button label a bit more space - 18 chars max */
 static char	mid_blank[] = "                  ";
 static Pixmap	mousefun_pm;
+static Pixmap	keybd_pm;
+
+/* popup message over button when mouse enters it */
+static void     mouse_balloon();
+static void     mouse_unballoon();
 
 void
 init_mousefun(tool)
-    TOOL	    tool;
+    Widget	    tool;
 {
     FirstArg(XtNwidth, MOUSEFUN_WD);
     /* start with nominal height and adjust later */
@@ -68,6 +79,71 @@ init_mousefun(tool)
 
     mousefun = XtCreateManagedWidget("mouse_panel", labelWidgetClass,
 				     tool, Args, ArgCount);
+    /* popup when mouse passes over button */
+    XtAddEventHandler(mousefun, EnterWindowMask, (Boolean) 0,
+		      mouse_balloon, (XtPointer) mousefun);
+    XtAddEventHandler(mousefun, LeaveWindowMask, (Boolean) 0,
+		      mouse_unballoon, (XtPointer) mousefun);
+}
+
+/* come here when the mouse passes over a button in the mouse indicator panel */
+
+static	Widget mouse_balloon_popup = (Widget) 0;
+
+static void
+mouse_balloon(widget, closure, event, continue_to_dispatch)
+    Widget        widget;
+    XtPointer	  closure;
+    XEvent*	  event;
+    Boolean*	  continue_to_dispatch;
+{
+	Widget	  box, balloon_label;
+	Position  x, y;
+	XtWidgetGeometry xtgeom,comp;
+	Dimension wpop;
+
+	if (!appres.show_balloons)
+	    return;
+
+	XtTranslateCoords(widget, 0, 0, &x, &y);
+	FirstArg(XtNx, x);
+	NextArg(XtNy, y);
+	mouse_balloon_popup = XtCreatePopupShell("mouse_balloon_popup",overrideShellWidgetClass,
+				tool, Args, ArgCount);
+	FirstArg(XtNborderWidth, 0);
+	NextArg(XtNhSpace, 0);
+	NextArg(XtNvSpace, 0);
+	box = XtCreateManagedWidget("box", boxWidgetClass, mouse_balloon_popup, Args, ArgCount);
+	FirstArg(XtNborderWidth, 0);
+	NextArg(XtNlabel, "Shows which mouse buttons\nare active in each mode");
+	balloon_label = XtCreateManagedWidget("label", labelWidgetClass,
+				    box, Args, ArgCount);
+
+	XtRealizeWidget(mouse_balloon_popup);
+	/* get width of popup with label in it */
+	FirstArg(XtNwidth, &wpop);
+	GetValues(balloon_label);
+	/* only change X position of widget */
+	xtgeom.request_mode = CWX;
+	/* shift popup left */
+	xtgeom.x = x-wpop-5;
+	(void) XtMakeGeometryRequest(mouse_balloon_popup, &xtgeom, &comp);
+	SetValues(balloon_label);
+	XtPopup(mouse_balloon_popup,XtGrabNone);
+}
+
+/* come here when the mouse leaves a button in the mouse panel */
+
+static void
+mouse_unballoon(widget, closure, event, continue_to_dispatch)
+    Widget          widget;
+    XtPointer	    closure;
+    XEvent*	    event;
+    Boolean*	    continue_to_dispatch;
+{
+    if (mouse_balloon_popup != (Widget) 0)
+	XtDestroyWidget(mouse_balloon_popup);
+    mouse_balloon_popup = 0;
 }
 
 static void
@@ -89,19 +165,41 @@ reset_mousefun()
     XSetForeground(tool_d, mouse_blank_gc, mouse_but_bg);
 
     mousefun_pm = XCreatePixmap(tool_d, XtWindow(mousefun),
-		    MOUSEFUN_WD, MOUSEFUN_HT, DefaultDepthOfScreen(tool_s));
+		    MOUSEFUN_WD, MOUSEFUN_HT, tool_dpth);
 
     XFillRectangle(tool_d, mousefun_pm, mouse_blank_gc, 0, 0,
 		   MOUSEFUN_WD, MOUSEFUN_HT);
 
+    /* draw the left button */
     XDrawRectangle(tool_d, mousefun_pm, mouse_button_gc, MOUSE_LEFT_SPACE,
 		   (int) (MOUSEFUN_HT * 0.45), MOUSE_BUT_WID, MOUSE_BUT_HGT);
+    /* draw a small line horizontally to the left of the left button */
+    XDrawLine(tool_d, mousefun_pm, mouse_button_gc, 
+		   MOUSE_LEFT_SPACE, (int) (MOUSEFUN_HT * 0.45 + MOUSE_BUT_HGT/2),
+		   MOUSE_LEFT_SPACE-5, (int) (MOUSEFUN_HT * 0.45 + MOUSE_BUT_HGT/2));
+
+    /* draw the middle button */
     XDrawRectangle(tool_d, mousefun_pm, mouse_button_gc,
 		   (int) (MOUSE_LEFT_SPACE + 1.5 * MOUSE_BUT_WID),
 		   (int) (MOUSEFUN_HT * 0.45), MOUSE_BUT_WID, MOUSE_BUT_HGT);
+    /* draw a small line vertically above the middle button */
+    XDrawLine(tool_d, mousefun_pm, mouse_button_gc, 
+		   (int) (MOUSE_LEFT_SPACE + 1.5 * MOUSE_BUT_WID + 0.5*MOUSE_BUT_WID),
+		   (int) (MOUSEFUN_HT * 0.45), 
+		   (int) (MOUSE_LEFT_SPACE + 1.5 * MOUSE_BUT_WID + 0.5*MOUSE_BUT_WID),
+		   (int) (MOUSEFUN_HT * 0.45 - 5)); 
+
+    /* draw the right button */
     XDrawRectangle(tool_d, mousefun_pm, mouse_button_gc,
 		   (int) (MOUSE_LEFT_SPACE + 3 * MOUSE_BUT_WID),
 		   (int) (MOUSEFUN_HT * 0.45), MOUSE_BUT_WID, MOUSE_BUT_HGT);
+    /* draw a small line horizontally to the right of the right button */
+    XDrawLine(tool_d, mousefun_pm, mouse_button_gc, 
+		   (int) (MOUSE_LEFT_SPACE + 4 * MOUSE_BUT_WID),
+		   (int) (MOUSEFUN_HT * 0.45 + MOUSE_BUT_HGT/2),
+		   (int) (MOUSE_LEFT_SPACE + 4 * MOUSE_BUT_WID)+5,
+		   (int) (MOUSEFUN_HT * 0.45 + MOUSE_BUT_HGT/2));
+
     FirstArg(XtNbackgroundPixmap, mousefun_pm);
     SetValues(mousefun);
     mouse_title();
@@ -134,7 +232,7 @@ setup_mousefun()
     SetValues(mousefun);
     XtManageChild(mousefun);
     reset_mousefun();
-    set_mousefun("", "", "");
+    set_mousefun("", "", "", "", "", "");
 }
 
 void
@@ -145,12 +243,16 @@ resize_mousefun()
 }
 
 void
-set_mousefun(left, middle, right)
+set_mousefun(left, middle, right, sh_left, sh_middle, sh_right)
     char	   *left, *middle, *right;
+    char	   *sh_left, *sh_middle, *sh_right;
 {
     strcpy(mousefun_l, left);
     strcpy(mousefun_m, middle);
     strcpy(mousefun_r, right);
+    strcpy(mousefun_sh_l, sh_left);
+    strcpy(mousefun_sh_m, sh_middle);
+    strcpy(mousefun_sh_r, sh_right);
 }
 
 #define KBD_POS_X 210
@@ -159,9 +261,16 @@ set_mousefun(left, middle, right)
 void
 draw_mousefun_kbd()
 {
-    XPutImage(tool_d, mousefun_pm, mouse_button_gc, &kbd_ic,
-              0, 0, MOUSEFUN_WD-10-kbd_ic.width, KBD_POS_Y, 
-	      kbd_ic.width, kbd_ic.height);
+    XGCValues	    values;
+    if (keybd_pm == 0) {
+	keybd_pm = XCreatePixmapFromBitmapData(tool_d, canvas_win, 
+				kbd_ic.bits, kbd_ic.width, kbd_ic.height, 
+				mouse_but_fg, mouse_but_bg, tool_dpth);
+    }
+    /* copy the keyboard image pixmap into the mouse function pixmap */
+    XCopyArea(tool_d, keybd_pm, mousefun_pm, mouse_button_gc,
+	      0, 0, kbd_ic.width, kbd_ic.height,
+	      MOUSEFUN_WD-10-kbd_ic.width, KBD_POS_Y);
     FirstArg(XtNbackgroundPixmap, 0);
     SetValues(mousefun);
     FirstArg(XtNbackgroundPixmap, mousefun_pm);
@@ -232,6 +341,12 @@ Cardinal *num_params;
 	draw_mousefun("Pan Up", "Drag", "Pan Down");
 }
 
+void
+draw_shift_mousefun_canvas()
+{
+    draw_mousefun(mousefun_sh_l, mousefun_sh_m, mousefun_sh_r);
+}
+
 void
 draw_mousefun_canvas()
 {
@@ -250,20 +365,34 @@ draw_mousefun_msg(s, xctr, ypos)
 		     xctr - (int) (width / 2), ypos, s, strlen(s));
 }
 
+static char	mousefun_cur_l[MOUSEFUN_MAX];
+static char	mousefun_cur_m[MOUSEFUN_MAX];
+static char	mousefun_cur_r[MOUSEFUN_MAX];
+
 void
 draw_mousefun(left, middle, right)
-    char	   *left, *middle, *right;
+    char     *left, *middle, *right;
 {
+  if (strcmp(left, mousefun_cur_l)
+      || strcmp(middle, mousefun_cur_m)
+      || strcmp(right, mousefun_cur_r)) {
     clear_mousefun();
     draw_mousefn2(left, middle, right);
+  }
 }
 
 #define MOUSE_LR_Y 32
 #define MOUSE_MID_Y 10
 void
 draw_mousefn2(left, middle, right)
-    char	   *left, *middle, *right;
+    char     *left, *middle, *right;
 {
+  if (strcmp(left, mousefun_cur_l)
+      || strcmp(middle, mousefun_cur_m)
+      || strcmp(right, mousefun_cur_r)) {
+    strcpy(mousefun_cur_l, left);
+    strcpy(mousefun_cur_m, middle);
+    strcpy(mousefun_cur_r, right);
     draw_mousefun_msg(left, MOUSE_LEFT_CTR, MOUSE_LR_Y);
     draw_mousefun_msg(middle, MOUSE_MID_CTR, MOUSE_MID_Y);
     draw_mousefun_msg(right, MOUSE_RIGHT_CTR, MOUSE_LR_Y);
@@ -271,6 +400,7 @@ draw_mousefn2(left, middle, right)
     SetValues(mousefun);
     FirstArg(XtNbackgroundPixmap, mousefun_pm);
     SetValues(mousefun);
+  }
 }
 
 void
@@ -333,7 +463,8 @@ String          kbd_translations =
 
 static int	actions_added=0;
 
-void init_kbd_actions()
+void
+init_kbd_actions()
 {
     if (!actions_added) {
 	actions_added = 1;
