@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -28,6 +31,7 @@
 #include "w_setup.h"
 #include "w_util.h"
 #include "w_zoom.h"
+#include "object.h"
 
 /*
  * The following will create rulers the same size as the initial screen size.
@@ -64,7 +68,7 @@ static unsigned char	tr_marker_bits[] = {
     0x80, 0x02,		/*       * *  */
     0x00, 0x01,		/*        *  */
 };
-icon_struct trm_pr = {TRM_WID, TRM_HT, tr_marker_bits};
+icon_struct trm_pr = {TRM_WID, TRM_HT, (char*)tr_marker_bits};
 
 static int	srroffx = 2, srroffy = -7;
 static unsigned char	srr_marker_bits[] = {
@@ -85,7 +89,7 @@ static unsigned char	srr_marker_bits[] = {
     0x80,		/*        *  */
     0x00
 };
-icon_struct srrm_pr = {SRM_WID, SRM_HT, srr_marker_bits};
+icon_struct srrm_pr = {SRM_WID, SRM_HT, (char*)srr_marker_bits};
 
 static int	srloffx = -10, srloffy = -7;
 static unsigned char	srl_marker_bits[] = {
@@ -106,7 +110,7 @@ static unsigned char	srl_marker_bits[] = {
     0x01,		/* *	      */
     0x00
 };
-icon_struct srlm_pr = {SRM_WID, SRM_HT, srl_marker_bits};
+icon_struct srlm_pr = {SRM_WID, SRM_HT, (char*)srl_marker_bits};
 
 static Pixmap	toparrow_pm = 0, sidearrow_pm = 0;
 static Pixmap	topruler_pm = 0, sideruler_pm = 0;
@@ -118,6 +122,10 @@ static		topruler_exposed();
 static		sideruler_selected();
 static		sideruler_exposed();
 
+/* popup message over button when mouse enters it */
+static void     unit_balloon();
+static void     unit_unballoon();
+
 redisplay_rulers()
 {
     redisplay_topruler();
@@ -161,16 +169,16 @@ static int	ONEMM = (PIX_PER_CM / 10);
 
 /************************* UNITBOX ************************/
 
-extern Widget   make_popup_menu();
-extern Atom     wm_delete_window;
-extern char    *panel_get_value();
+extern Atom	wm_delete_window;
+extern char	*panel_get_value();
 void popup_unit_panel();
-static int      rul_unit_setting, fig_unit_setting=0, fig_scale_setting=0;
-static Widget   rul_unit_panel, fig_unit_panel, fig_scale_panel;
-static Widget   rul_unit_menu, fig_unit_menu, fig_scale_menu;
-static Widget   scale_factor_lab, scale_factor_panel;
-static Widget   user_unit_lab, user_unit_panel;
-
+static Boolean	rul_unit_setting;
+static int	fig_unit_setting=0, fig_scale_setting=0;
+static Widget	rul_unit_panel, fig_unit_panel, fig_scale_panel;
+static Widget	rul_unit_menu, fig_unit_menu, fig_scale_menu;
+static Widget	scale_factor_lab, scale_factor_panel;
+static Widget	user_unit_lab, user_unit_panel;
+static void     unit_panel_cancel(), unit_panel_set();
 
 XtActionsRec	unitbox_actions[] =
 {
@@ -181,10 +189,20 @@ XtActionsRec	unitbox_actions[] =
 };
 
 static String	unitbox_translations =
-"<EnterWindow>:EnterUnitBox()\n\
-    <LeaveWindow>:LeaveUnitBox()\n\
-    <Btn1Down>:HomeRulers()\n\
-    <Btn3Down>:PopupUnits()\n";
+	"<EnterWindow>:EnterUnitBox()\n\
+	<LeaveWindow>:LeaveUnitBox()\n\
+	<Btn1Down>:HomeRulers()\n\
+	<Btn3Down>:PopupUnits()\n";
+
+static String   unit_translations =
+        "<Message>WM_PROTOCOLS: QuitUnits()\n\
+	<Key>Return:SetUnits()\n";
+
+static XtActionsRec     unit_actions[] =
+{
+    {"QuitUnits", (XtActionProc) unit_panel_cancel},
+    {"SetUnits", (XtActionProc) unit_panel_set},
+};
 
 int
 init_unitbox(tool)
@@ -216,22 +234,119 @@ init_unitbox(tool)
     unitbox_sw = XtCreateWidget("unitbox", labelWidgetClass, tool,
 				Args, ArgCount);
     XtAppAddActions(tool_app, unitbox_actions, XtNumber(unitbox_actions));
+    /* popup when mouse passes over button */
+    XtAddEventHandler(unitbox_sw, EnterWindowMask, (Boolean) 0,
+		      unit_balloon, (XtPointer) unitbox_sw);
+    XtAddEventHandler(unitbox_sw, LeaveWindowMask, (Boolean) 0,
+		      unit_unballoon, (XtPointer) unitbox_sw);
     XtOverrideTranslations(unitbox_sw,
 			   XtParseTranslationTable(unitbox_translations));
     return (1);
 }
 
-static String   unit_translations =
-        "<Message>WM_PROTOCOLS: QuitUnits()\n";
-static void     unit_panel_cancel(), unit_panel_set();
-static XtActionsRec     unit_actions[] =
-{
-    {"QuitUnits", (XtActionProc) unit_panel_cancel},
-    {"SetUnit", (XtActionProc) unit_panel_set},
-};
+static	Pixmap mouse_l=(Pixmap) 0, mouse_r=(Pixmap) 0;
+
+#define mouse_r_width 19
+#define mouse_r_height 10
+static char mouse_r_bits[] = {
+   0xff, 0xff, 0x07, 0x41, 0xf0, 0x07, 0x41, 0xf0, 0x07, 0x41, 0xf0, 0x07,
+   0x41, 0xf0, 0x07, 0x41, 0xf0, 0x07, 0x41, 0xf0, 0x07, 0x41, 0xf0, 0x07,
+   0x41, 0xf0, 0x07, 0xff, 0xff, 0x07};
+
+#define mouse_l_width 19
+#define mouse_l_height 10
+static char mouse_l_bits[] = {
+   0xff, 0xff, 0x07, 0x7f, 0x10, 0x04, 0x7f, 0x10, 0x04, 0x7f, 0x10, 0x04,
+   0x7f, 0x10, 0x04, 0x7f, 0x10, 0x04, 0x7f, 0x10, 0x04, 0x7f, 0x10, 0x04,
+   0x7f, 0x10, 0x04, 0xff, 0xff, 0x07};
+
 
 static Widget	unit_popup, form, cancel, set, beside, below, label;
 
+/* come here when the mouse passes over the unit box */
+
+static	Widget unit_balloon_popup = (Widget) 0;
+
+static void
+unit_balloon(widget, closure, event, continue_to_dispatch)
+    Widget        widget;
+    XtPointer	  closure;
+    XEvent*	  event;
+    Boolean*	  continue_to_dispatch;
+{
+	Widget	  box, balloon_label;
+	Position  x, y;
+
+	if (!appres.show_balloons)
+	    return;
+
+	/* create the bitmaps that look like mouse buttons pressed */
+	if (mouse_l == 0) {
+	    mouse_l = XCreateBitmapFromData(tool_d, tool_w, 
+				mouse_l_bits, mouse_l_width, mouse_l_height);
+	    mouse_r = XCreateBitmapFromData(tool_d, tool_w, 
+				mouse_r_bits, mouse_r_width, mouse_r_height);
+	}
+
+	XtTranslateCoords(widget, 0, 0, &x, &y);
+	/* if mode panel is on right (units on left) position popup to right of box */
+	if (appres.RHS_PANEL) {
+	    x += SIDERULER_WD+5;
+	}
+	FirstArg(XtNx, x);
+	NextArg(XtNy, y);
+	unit_balloon_popup = XtCreatePopupShell("unit_balloon_popup",overrideShellWidgetClass,
+				tool, Args, ArgCount);
+	FirstArg(XtNborderWidth, 0);
+	NextArg(XtNhSpace, 0);
+	NextArg(XtNvSpace, 0);
+	box = XtCreateManagedWidget("box", boxWidgetClass, unit_balloon_popup, Args, ArgCount);
+
+	/* now make two label widgets, one for left button and one for right */
+	FirstArg(XtNborderWidth, 0);
+	NextArg(XtNleftBitmap, mouse_l);	/* bitmap of mouse with left button pushed */
+	NextArg(XtNlabel, "Pan to (0,0)   ");
+	balloon_label = XtCreateManagedWidget("l_label", labelWidgetClass,
+				    box, Args, ArgCount);
+	FirstArg(XtNborderWidth, 0);
+	NextArg(XtNleftBitmap, mouse_r);	/* bitmap of mouse with right button pushed */
+	NextArg(XtNlabel, "Set Units/Scale");
+	balloon_label = XtCreateManagedWidget("r_label", labelWidgetClass,
+				    box, Args, ArgCount);
+	XtRealizeWidget(unit_balloon_popup);
+	/* if the mode panel is on the left-hand side (units on right)
+	   shift popup to the left */
+	if (!appres.RHS_PANEL) {
+	    XtWidgetGeometry xtgeom,comp;
+	    Dimension wpop;
+
+	    /* get width of popup with label in it */
+	    FirstArg(XtNwidth, &wpop);
+	    GetValues(balloon_label);
+	    /* only change X position of widget */
+	    xtgeom.request_mode = CWX;
+	    /* shift popup left */
+	    xtgeom.x = x-wpop-5;
+	    (void) XtMakeGeometryRequest(unit_balloon_popup, &xtgeom, &comp);
+	    SetValues(balloon_label);
+	}
+	XtPopup(unit_balloon_popup,XtGrabNone);
+}
+
+/* come here when the mouse leaves a button in the mode panel */
+
+static void
+unit_unballoon(widget, closure, event, continue_to_dispatch)
+    Widget          widget;
+    XtPointer	    closure;
+    XEvent*	    event;
+    Boolean*	    continue_to_dispatch;
+{
+    if (unit_balloon_popup != (Widget) 0)
+	XtDestroyWidget(unit_balloon_popup);
+    unit_balloon_popup = 0;
+}
+
 /* handle unit/scale settings */
 
 static void
@@ -257,7 +372,7 @@ unit_panel_set(w, ev)
     int old_rul_unit;
 
     old_rul_unit = appres.INCHES;
-    appres.INCHES = rul_unit_setting ? 1 : 0;
+    appres.INCHES = rul_unit_setting ? True : False;
     init_grid();	/* change point positioning messages if necessary */
     if (fig_scale_setting)
 	appres.user_scale = (float) atof(panel_get_value(scale_factor_panel));
@@ -267,7 +382,7 @@ unit_panel_set(w, ev)
     if (fig_unit_setting) {
         strncpy(cur_fig_units,
 	    panel_get_value(user_unit_panel),
-	    sizeof(cur_fig_units));
+	    sizeof(cur_fig_units)-1);
 	put_msg("Ruler scale: %s,  Figure scale: 1 %s = %.4f %s",
 		appres.INCHES ? "in" : "cm",
 		appres.INCHES ? "in" : "cm",
@@ -284,6 +399,13 @@ unit_panel_set(w, ev)
 	SetValues(unitbox_sw);
 	reset_rulers();
 	setup_grid(cur_gridmode);
+	if(!emptyfigure()) { /* rescale, HWS */
+	  if(old_rul_unit)
+	    read_scale_compound(&objects,(2.54*PIX_PER_CM)/((float)PIX_PER_INCH),0);
+	  else
+	    read_scale_compound(&objects,((float)PIX_PER_INCH)/(2.54*PIX_PER_CM),0);
+	  redisplay_canvas();
+	}
     }
     unit_panel_dismiss();
 }
@@ -323,7 +445,7 @@ rul_unit_select(w, new_unit, garbage)
 {
     FirstArg(XtNlabel, XtName(w));
     SetValues(rul_unit_panel);
-    rul_unit_setting = (int) new_unit;
+    rul_unit_setting = ((int)new_unit)==1? True: False;
     if (rul_unit_setting)
 	put_msg("ruler scale : inches");
     else
@@ -343,12 +465,18 @@ popup_unit_panel()
     "Ruler units ", "User defined"};
     static char    *fig_scale_items[] = {
     "Unity       ", "User defined"};
+    int		    x;
 
     FirstArg(XtNwidth, &width);
     NextArg(XtNheight, &height);
     GetValues(tool);
-    /* position the popup 2/3 in from left and 1/3 down from top */
-    XtTranslateCoords(tool, (Position) (2 * width / 3), (Position) (height / 3),
+    /* position the popup 2/3 in from left (1/8 if mode panel on right) 
+       and 1/3 down from top */
+    if (appres.RHS_PANEL)
+	x = (width / 8);
+    else
+	x = (2 * width / 3);
+    XtTranslateCoords(tool, (Position) x, (Position) (height / 6),
 		      &x_val, &y_val);
 
     FirstArg(XtNx, x_val);
@@ -374,7 +502,7 @@ popup_unit_panel()
 
     /* make ruler units menu */
 
-    rul_unit_setting = appres.INCHES ? 1 : 0;
+    rul_unit_setting = appres.INCHES ? True : False;
     FirstArg(XtNfromVert, label);
     NextArg(XtNborderWidth, 0);
     beside = XtCreateManagedWidget("Ruler Units  =", labelWidgetClass,
@@ -382,7 +510,7 @@ popup_unit_panel()
 
     FirstArg(XtNfromVert, label);
     NextArg(XtNfromHoriz, beside);
-    rul_unit_panel = XtCreateManagedWidget(rul_unit_items[rul_unit_setting],
+    rul_unit_panel = XtCreateManagedWidget(rul_unit_items[rul_unit_setting? 1:0],
 				menuButtonWidgetClass, form, Args, ArgCount);
     below = rul_unit_panel;
     rul_unit_menu = make_popup_menu(rul_unit_items, XtNumber(rul_unit_items),
@@ -672,8 +800,7 @@ setup_topruler()
     /* make pixmaps for top ruler arrow */
     toparrow_pm = XCreatePixmapFromBitmapData(tool_d, topruler_win, 
 				trm_pr.bits, trm_pr.width, trm_pr.height, 
-				fg^bg, (unsigned long) 0,
-				DefaultDepthOfScreen(tool_s));
+				fg^bg, (unsigned long) 0, tool_dpth);
 
     /* now make the real xor gc */
     gcv.background = bg;
@@ -683,8 +810,7 @@ setup_topruler()
     XDefineCursor(tool_d, topruler_win, lr_arrow_cursor);
 
     topruler_pm = XCreatePixmap(tool_d, topruler_win,
-				TOPRULER_WD, TOPRULER_HT,
-				DefaultDepthOfScreen(tool_s));
+				TOPRULER_WD, TOPRULER_HT, tool_dpth);
 
     reset_topruler();
 }
@@ -693,17 +819,16 @@ resize_topruler()
 {
     XFreePixmap(tool_d, topruler_pm);
     topruler_pm = XCreatePixmap(tool_d, topruler_win,
-				TOPRULER_WD, TOPRULER_HT,
-				DefaultDepthOfScreen(tool_s));
+				TOPRULER_WD, TOPRULER_HT, tool_dpth);
 
     reset_topruler();
 }
 
 reset_topruler()
 {
-    register int    i, j;
+    register int    i;
     register Pixmap p = topruler_pm;
-    char	    number[4],clen;
+    char	    number[6],clen;
     int		    X0;
     float	    skip;
 
@@ -726,45 +851,43 @@ reset_topruler()
     X0 = BACKX(0);
     if (appres.INCHES) {
 	X0 -= (X0 % SINCH);
-	for (i = X0+SINCH-1; i <= X0+round(TOPRULER_WD/zoomscale); i += SINCH) {
-	    j = i + 1;
-	    if ((int)(j/skip) % PIX_PER_INCH == 0) {
-		if (1.0*j/PIX_PER_INCH == (int)(j/PIX_PER_INCH))
-		    sprintf(number, "%-d", (int)(j / PIX_PER_INCH));
+	for (i = X0; i <= X0+round(TOPRULER_WD/zoomscale); i += SINCH) {
+	    if ((int)(i/skip) % PIX_PER_INCH == 0) {
+		if (1.0*i/PIX_PER_INCH == (int)(i/PIX_PER_INCH))
+		    sprintf(number, "%-d", (int)(i / PIX_PER_INCH));
 		else
-		    sprintf(number, "%-.1f", 1.0 * j / PIX_PER_INCH);
+		    sprintf(number, "%-.1f", (float)(1.0 * i / PIX_PER_INCH));
 		XDrawString(tool_d, p, tr_gc, ZOOMX(i) - strlen(number)*clen/2,
 			TOPRULER_HT - INCH_MARK - 5, number, strlen(number));
 	    }
-	    if (j % PIX_PER_INCH == 0) {
+	    if (i % PIX_PER_INCH == 0) {
 		XDrawLine(tool_d, p, tr_gc, ZOOMX(i), TOPRULER_HT - 1, ZOOMX(i),
 			  TOPRULER_HT - INCH_MARK - 1);
-	    } else if (j % HINCH == 0)
+	    } else if (i % HINCH == 0)
 		XDrawLine(tool_d, p, tr_gc, ZOOMX(i), TOPRULER_HT - 1, ZOOMX(i),
 			  TOPRULER_HT - HALF_MARK - 1);
-	    else if (j % QINCH == 0 && display_zoomscale >= 0.2)
+	    else if (i % QINCH == 0 && display_zoomscale >= 0.2)
 		XDrawLine(tool_d, p, tr_gc, ZOOMX(i), TOPRULER_HT - 1, ZOOMX(i),
 			  TOPRULER_HT - QUARTER_MARK - 1);
-	    else if (j % SINCH == 0 && display_zoomscale >= 0.4)
+	    else if (i % SINCH == 0 && display_zoomscale >= 0.4)
 		XDrawLine(tool_d, p, tr_gc, ZOOMX(i), TOPRULER_HT - 1, ZOOMX(i),
 			  TOPRULER_HT - SIXTEENTH_MARK - 1);
 	}
     } else {
 	X0 -= (X0 % TWOMM);
-	for (i = X0+TWOMM-1; i <= X0+round(TOPRULER_WD/zoomscale); i += ONEMM) {
-	    j = i + 1;
-	    if ((int)(j/skip) % PIX_PER_CM == 0) {
-		if (1.0*j/PIX_PER_CM == (int)(j/PIX_PER_CM))
-		    sprintf(number, "%-d", (int)(j / PIX_PER_CM));
+	for (i = X0; i <= X0+round(TOPRULER_WD/zoomscale); i += ONEMM) {
+	    if ((int)(i/skip) % PIX_PER_CM == 0) {
+		if (1.0*i/PIX_PER_CM == (int)(i/PIX_PER_CM))
+		    sprintf(number, "%-d", (int)(i / PIX_PER_CM));
 		else
-		    sprintf(number, "%-.1f", 1.0 * j / PIX_PER_CM);
+		    sprintf(number, "%-.1f", (float)(1.0 * i / PIX_PER_CM));
 		XDrawString(tool_d, p, tr_gc, ZOOMX(i) - strlen(number)*clen/2,
 			TOPRULER_HT - INCH_MARK - 5, number, strlen(number));
 	    }
-	    if (j % PIX_PER_CM == 0)
+	    if (i % PIX_PER_CM == 0)
 		XDrawLine(tool_d, p, tr_gc, ZOOMX(i), TOPRULER_HT - 1, ZOOMX(i),
 			  TOPRULER_HT - INCH_MARK - 1);
-	    else if (j % TWOMM == 0 && display_zoomscale >= 0.3)
+	    else if (i % TWOMM == 0 && display_zoomscale >= 0.3)
 		XDrawLine(tool_d, p, tr_gc, ZOOMX(i), TOPRULER_HT - 1, ZOOMX(i),
 			  TOPRULER_HT - QUARTER_MARK - 1);
 	}
@@ -938,13 +1061,11 @@ setup_sideruler()
     if (appres.RHS_PANEL) {
 	sidearrow_pm = XCreatePixmapFromBitmapData(tool_d, sideruler_win, 
 				srlm_pr.bits, srlm_pr.width, srlm_pr.height, 
-				fg^bg, (unsigned long) 0,
-				DefaultDepthOfScreen(tool_s));
+				fg^bg, (unsigned long) 0, tool_dpth);
     } else {
 	sidearrow_pm = XCreatePixmapFromBitmapData(tool_d, sideruler_win, 
 				srrm_pr.bits, srrm_pr.width, srrm_pr.height, 
-				fg^bg, (unsigned long) 0,
-				DefaultDepthOfScreen(tool_s));
+				fg^bg, (unsigned long) 0, tool_dpth);
     }
 
     /* now make the real xor gc */
@@ -955,8 +1076,7 @@ setup_sideruler()
     XDefineCursor(tool_d, sideruler_win, ud_arrow_cursor);
 
     sideruler_pm = XCreatePixmap(tool_d, sideruler_win,
-				 SIDERULER_WD, SIDERULER_HT,
-				 DefaultDepthOfScreen(tool_s));
+				 SIDERULER_WD, SIDERULER_HT, tool_dpth);
 
     reset_sideruler();
 }
@@ -965,16 +1085,15 @@ resize_sideruler()
 {
     XFreePixmap(tool_d, sideruler_pm);
     sideruler_pm = XCreatePixmap(tool_d, sideruler_win,
-				 SIDERULER_WD, SIDERULER_HT,
-				 DefaultDepthOfScreen(tool_s));
+				 SIDERULER_WD, SIDERULER_HT, tool_dpth);
     reset_sideruler();
 }
 
 reset_sideruler()
 {
-    register int    i, j;
+    register int    i;
     register Pixmap p = sideruler_pm;
-    char	    number[4];
+    char	    number[6];
     int		    Y0;
     float	    skip;
 
@@ -998,54 +1117,52 @@ reset_sideruler()
     if (appres.INCHES) {
 	Y0 -= (Y0 % SINCH);
 	if (appres.RHS_PANEL) {
-	    for (i = Y0+SINCH-1; i <= Y0+round(SIDERULER_HT/zoomscale); i += SINCH) {
-		j = i + 1;
-		if (j % PIX_PER_INCH == 0) {
+	    for (i = Y0; i <= Y0+round(SIDERULER_HT/zoomscale); i += SINCH) {
+		if (i % PIX_PER_INCH == 0) {
 		    XDrawLine(tool_d, p, sr_gc, SIDERULER_WD - INCH_MARK,
 			      ZOOMY(i), SIDERULER_WD, ZOOMY(i));
-		    if ((int)(j/skip) % PIX_PER_INCH == 0) {
-			if (1.0*j/PIX_PER_INCH == (int)(j/PIX_PER_INCH))
-			    sprintf(number, "%d", (int)(j / PIX_PER_INCH));
+		    if ((int)(i/skip) % PIX_PER_INCH == 0) {
+			if (1.0*i/PIX_PER_INCH == (int)(i / PIX_PER_INCH))
+			    sprintf(number, "%d", (int)(i / PIX_PER_INCH));
 			else
-			    sprintf(number, "%.1f", j / PIX_PER_INCH);
+			    sprintf(number, "%.1f", (float)(1.0 * i / PIX_PER_INCH));
 			XDrawString(tool_d, p, sr_gc,
 				SIDERULER_WD - INCH_MARK - 22, ZOOMY(i) + 3,
 				number, strlen(number));
 		    }
-		} else if (j % HINCH == 0)
+		} else if (i % HINCH == 0)
 		    XDrawLine(tool_d, p, sr_gc,
 			      SIDERULER_WD - HALF_MARK, ZOOMY(i),
 			      SIDERULER_WD, ZOOMY(i));
-		else if (j % QINCH == 0 && display_zoomscale >= 0.2)
+		else if (i % QINCH == 0 && display_zoomscale >= 0.2)
 		    XDrawLine(tool_d, p, sr_gc,
 			      SIDERULER_WD - QUARTER_MARK, ZOOMY(i),
 			      SIDERULER_WD, ZOOMY(i));
-		else if (j % SINCH == 0 && display_zoomscale >= 0.4)
+		else if (i % SINCH == 0 && display_zoomscale >= 0.4)
 		    XDrawLine(tool_d, p, sr_gc,
 			      SIDERULER_WD - SIXTEENTH_MARK, ZOOMY(i),
 			      SIDERULER_WD, ZOOMY(i));
 	    }
 	} else {
-	    for (i = Y0+SINCH-1; i <= Y0+round(SIDERULER_HT/zoomscale); i += SINCH) {
-		j = i + 1;
-		if (j % PIX_PER_INCH == 0) {
+	    for (i = Y0; i <= Y0+round(SIDERULER_HT/zoomscale); i += SINCH) {
+		if (i % PIX_PER_INCH == 0) {
 		    XDrawLine(tool_d, p, sr_gc, 0, ZOOMY(i),
 			      INCH_MARK - 1, ZOOMY(i));
-		    if ((int)(j/skip) % PIX_PER_INCH == 0) {
-			if (1.0*j/PIX_PER_INCH == (int)(j/PIX_PER_INCH))
-			    sprintf(number, "%d", (int)(j / PIX_PER_INCH));
+		    if ((int)(i/skip) % PIX_PER_INCH == 0) {
+			if (1.0*i/PIX_PER_INCH == (int)(i / PIX_PER_INCH))
+			    sprintf(number, "%d", (int)(i / PIX_PER_INCH));
 			else
-			    sprintf(number, "%.1f", j / PIX_PER_INCH);
+			    sprintf(number, "%.1f", (float)(1.0 * i / PIX_PER_INCH));
 		        XDrawString(tool_d, p, sr_gc, INCH_MARK + 3,
 				ZOOMY(i) + 3, number, strlen(number));
 		    }
-		} else if (j % HINCH == 0)
+		} else if (i % HINCH == 0)
 		    XDrawLine(tool_d, p, sr_gc, 0, ZOOMY(i),
 			      HALF_MARK - 1, ZOOMY(i));
-		else if (j % QINCH == 0 && display_zoomscale >= 0.2)
+		else if (i % QINCH == 0 && display_zoomscale >= 0.2)
 		    XDrawLine(tool_d, p, sr_gc, 0, ZOOMY(i),
 			      QUARTER_MARK - 1, ZOOMY(i));
-		else if (j % SINCH == 0 && display_zoomscale >= 0.4)
+		else if (i % SINCH == 0 && display_zoomscale >= 0.4)
 		    XDrawLine(tool_d, p, sr_gc, 0, ZOOMY(i),
 			      SIXTEENTH_MARK - 1, ZOOMY(i));
 	    }
@@ -1053,40 +1170,38 @@ reset_sideruler()
     } else {
 	Y0 -= (Y0 % TWOMM);
 	if (appres.RHS_PANEL) {
-	    for (i = Y0+TWOMM-1; i <= Y0+round(SIDERULER_HT/zoomscale); i++) {
-		j = i + 1;
-		if (j % PIX_PER_CM == 0) {
+	    for (i = Y0; i <= Y0+round(SIDERULER_HT/zoomscale); i++) {
+		if (i % PIX_PER_CM == 0) {
 		    XDrawLine(tool_d, p, sr_gc, SIDERULER_WD - INCH_MARK,
 			      ZOOMY(i), SIDERULER_WD, ZOOMY(i));
-		    if ((int)(j/skip) % PIX_PER_CM == 0) {
-			if (1.0*j/PIX_PER_CM == (int)(j/PIX_PER_CM))
-			    sprintf(number, "%d", (int)(j / PIX_PER_CM));
+		    if ((int)(i/skip) % PIX_PER_CM == 0) {
+			if (1.0*i/PIX_PER_CM == (int)(i / PIX_PER_CM))
+			    sprintf(number, "%d", (int)(i / PIX_PER_CM));
 			else
-			    sprintf(number, "%.1f", j / PIX_PER_CM);
+			    sprintf(number, "%.1f", (float)(1.0 * i / PIX_PER_CM));
 			XDrawString(tool_d, p, sr_gc,
 				SIDERULER_WD - INCH_MARK - 14, ZOOMY(i) + 3,
 				number, strlen(number));
 		    }
-		} else if (j % TWOMM == 0 && display_zoomscale >= 0.3)
+		} else if (i % TWOMM == 0 && display_zoomscale >= 0.3)
 		    XDrawLine(tool_d, p, sr_gc,
 			      SIDERULER_WD - QUARTER_MARK, ZOOMY(i),
 			      SIDERULER_WD, ZOOMY(i));
 	    }
 	} else {
-	    for (i = Y0+TWOMM-1; i <= Y0+round(SIDERULER_HT/zoomscale); i++) {
-		j = i + 1;
-		if (j % PIX_PER_CM == 0) {
+	    for (i = Y0; i <= Y0+round(SIDERULER_HT/zoomscale); i++) {
+		if (i % PIX_PER_CM == 0) {
 		    XDrawLine(tool_d, p, sr_gc, 0, ZOOMY(i),
 			      INCH_MARK - 1, ZOOMY(i));
-		    if ((int)(j/skip) % PIX_PER_CM == 0) {
-			if (1.0*j/PIX_PER_CM == (int)(j/PIX_PER_CM))
-			    sprintf(number, "%d", (int)(j / PIX_PER_CM));
+		    if ((int)(i/skip) % PIX_PER_CM == 0) {
+			if (1.0*i/PIX_PER_CM == (int)(i / PIX_PER_CM))
+			    sprintf(number, "%d", (int)(i / PIX_PER_CM));
 			else
-			    sprintf(number, "%.1f", j / PIX_PER_CM);
+			    sprintf(number, "%.1f", (float)(1.0 * i / PIX_PER_CM));
 			XDrawString(tool_d, p, sr_gc, INCH_MARK + 3,
 				ZOOMY(i) + 3, number, strlen(number));
 		    }
-		} else if (j % TWOMM == 0 && display_zoomscale >= 0.3)
+		} else if (i % TWOMM == 0 && display_zoomscale >= 0.3)
 		    XDrawLine(tool_d, p, sr_gc, 0, ZOOMY(i),
 			      QUARTER_MARK - 1, ZOOMY(i));
 	    }
