@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include <X11/keysym.h>
@@ -48,8 +51,8 @@ static int	(*save_middlebut_proc) ();
 static int	(*save_rightbut_proc) ();
 static int	save_action_on;
 
-float		display_zoomscale=1.0;
-float		zoomscale = 1.0/ZOOM_FACTOR;
+float		display_zoomscale;	/* both zoomscales initialized in main() */
+float		zoomscale;
 int		zoomxoff = 0;
 int		zoomyoff = 0;
 
@@ -61,6 +64,16 @@ zoom_selected(x, y, button)
     int		    x, y;
     unsigned int    button;
 {
+    /* if trying to zoom while drawing an object, don't allow it */
+    if (action_on && button != Button2) {	/* panning is ok */
+	if (cur_mode == F_TEXT)
+	    finish_text_input();		/* finish up any text input */
+	else {
+	    put_msg("Finish (or cancel) the current operation before zooming");
+	    beep();
+	    return;
+	}
+    }
     if (!zoom_in_progress) {
 	switch (button) {
 	case Button1:
@@ -98,6 +111,15 @@ static
 init_zoombox_drawing(x, y)
     int		    x, y;
 {
+    if (action_on) {
+	if (cur_mode == F_TEXT)
+	    finish_text_input();	/* finish up any text input */
+	else {
+	    put_msg("Finish (or cancel) the current operation before zooming");
+	    beep();
+	    return;
+	}
+    }
     save_kbd_proc = canvas_kbd_proc;
     save_locmove_proc = canvas_locmove_proc;
     save_leftbut_proc = canvas_leftbut_proc;
